/*
 * Amza Elena Violeta 
 * Singer Mood Lamp
 *  
 * PM - 2018
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <util/delay.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "usart.h"

/* dimesiune buffer de primire comenzi pe USART0*/
#define BUFLEN 50
/* red  PB3(OCR0A) green - PB4(OCR0B) blue - PD7(OCR2A)*/
#define RED(x) OCR0A = (x);
#define GREEN(x) OCR0B = (x);
#define BLUE(x) OCR2A = (x);
/* BUZZER PORT si PIN*/
#define BUZZER_PORT PORTA
#define BUZZER PA0

/* calucularea notelor asociate*/
/* calcularea urmatoarei valori pentru culorile din spectrul unui mod*/
#define NOTA_READ 20
#define NOTA_WORK 10
#define NOTA_SLEEP 250
#define NOTA_WAKE_UP 50
#define NOTA_DEFAULT 100
#define INTERVAL_READ 2
#define INTERVAL_WORK 20
#define INTERVAL_SLEEP 1
#define INTERVAL_DEFAULT 7
#define INTERVAL_WAKE_UP 10
#define FREQUENCY_ONE_COLOR 1000

char c;
char value[4];
char buff[BUFLEN];
int on =  1;
int i_color = 0;
int buff_i = 0;
int music_activ = 0;
int wake_up_mode = 0;
int work_mode = 0;
int sleep_mode = 0;
int read_mode = 0;
int default_mode = 0;
int nota = 100;

void IO_init() {
    DDRD |= (1 << PD7);
    DDRB |= (1 << PB3) | (1 << PB4);
    DDRA |= (1 << PA0);
    PORTA |= (1 << PA0);

}
/* intializari pentru PWM*/
void PWM_Init()
{
    TCCR0A = (3 << WGM00) | (2 << COM0A0) | (2 << COM0B0);
    TCCR0B = (1 << CS00);

    TCCR2A = (3 << WGM20) | (2 << COM2A0);
    TCCR2B = (1 << CS20);

    OCR0A = 0x00;
    OCR0B = 0x00;
    OCR2A = 0x00;
}

/* stinge lampa*/
void end (){
    BLUE(0);
    GREEN(0);
    RED(0);
    on = 0;
}

/* led - stins*/
void reset (){
    BLUE(0);
    GREEN(0);
    RED(0);
}

/* reproduce nota asociata culorii*/
void sing_nota(long frequency) 
{
    long numCycles, i;
    double delayValue;
 
    delayValue = 1000000 / frequency / 2;
    numCycles = frequency * 0.05;
    for (i = 0; i < 10; i++) { 
        BUZZER_PORT &= ~(1 << BUZZER); 
        _delay_us(delayValue); 
        BUZZER_PORT |= (1 << BUZZER); 
        _delay_us(delayValue);
    }
}

/* se trateza o comanda primita */
void treat_commad() {
    buff_i = 0;
    i_color = 0;
    if (on == 1 && strncmp(buff,"rgb",3) == 0) {
        nota = 100;
        strncpy(value, buff + 3, 3);
        value[3] = '\0';
        c = (char) atoi(value);
        nota += atoi(value);
        RED(c);
        strncpy(value, buff + 6, 3);
        value[3] = '\0';
        c = (char) atoi(value);
        nota += atoi(value);
        GREEN(c);
        strncpy(value, buff + 9, 3);
        value[3] = '\0';
        c = (char) atoi(value);
        BLUE(c);
        nota += atoi(value);
        if (music_activ == 1) {
            sing_nota(nota);
        }
    } else if (on == 1 &&  strncmp(buff,"music",5) == 0) {
        music_activ = 1;
    } else if (on == 1 && strncmp(buff,"off",3) == 0) {
        end();
    } else if (on == 1 && strncmp(buff,"not_music",9) == 0) {
        music_activ = 0;
    } else if (on == 1 && strncmp(buff,"wake_up",7) == 0) {
        wake_up_mode = 1;
        reset ();
    } else if (on == 1 && strncmp(buff,"not_wake_up",11) == 0) {
        wake_up_mode = 0;
        reset ();
    } else if (on == 1 && strncmp(buff,"work",4) == 0) {
        work_mode = 1;
        reset ();
    } else if (on == 1 && strncmp(buff,"not_work",8) == 0) {
        work_mode = 0;
        reset ();
    } else if (on == 1 && strncmp(buff,"sleep",5) == 0) {
        sleep_mode = 1;
        reset ();
    } else if (on == 1 && strncmp(buff,"not_sleep",9) == 0) {
        sleep_mode = 0;
        reset ();
    } else if (on == 1 && strncmp(buff,"read",4) == 0) {
        read_mode = 1;
        reset ();
    } else if (on == 1 && strncmp(buff,"not_read",8) == 0) {
        read_mode = 0;
        reset ();
    } else if (on == 1 && strncmp(buff,"default",7) == 0) {
        default_mode = 1;
        reset ();
    } else if (on == 1 && strncmp(buff,"not_default",11) == 0) {
        default_mode = 0;
        reset ();
    } else if (strncmp(buff,"on_lamp",7) == 0) {
        on = 1;
    }
}

/* intrerupere primita de Bluetooth */
ISR (USART0_RX_vect) {
    buff[buff_i++] = UDR0;

    /* caracter final al comenzii*/
    if(buff[buff_i - 1] == '?') {
        treat_commad();
    }
}

/* se calculeaza urmatoarea culoare din spectrul asociat modul-ului curent de functionare a lampii*/
void next_color() {
   
    c = (char) (i_color % 255);
    if (wake_up_mode == 1) {          
        RED(c);
        c = (char) (i_color % 50);
        GREEN(c);
        nota = (NOTA_WAKE_UP + i_color) % FREQUENCY_ONE_COLOR;
        if (music_activ == 1) {
            sing_nota(nota);
        }
        i_color += INTERVAL_WAKE_UP;
    } else if(sleep_mode == 1){
        BLUE(c);
        c = (char) (i_color % 70);
        GREEN(c);
        nota = (NOTA_SLEEP + i_color) % FREQUENCY_ONE_COLOR;
        if (music_activ == 1) {
            sing_nota(nota);
        }
        i_color += INTERVAL_SLEEP;
    } else if (work_mode == 1) {
        GREEN(c);
        c = (char) (i_color % 50);
        RED(c);
        nota = (NOTA_WORK+ i_color) % FREQUENCY_ONE_COLOR;
        if (music_activ == 1) {
            sing_nota(nota);
        }
        i_color += INTERVAL_WORK;
    } else if (read_mode == 1) {
        RED(c);
        c = (char) (i_color % 100);
        GREEN(c);
        nota = (NOTA_READ + i_color) % (2 * FREQUENCY_ONE_COLOR);
        if (music_activ == 1) {
            sing_nota(nota);
        }
        i_color += INTERVAL_READ;
    } else if (default_mode == 1) {
        RED(c);
        c = (char) (i_color % 100);
        GREEN(c);
        c = (char) (i_color % 200);
        BLUE(c);
        nota = (NOTA_DEFAULT + i_color) %  (3 * FREQUENCY_ONE_COLOR);
        if (music_activ == 1) {
            sing_nota(nota);
        }
        i_color += INTERVAL_DEFAULT;
    }
}
int main(void) 
{   
    IO_init();
    PWM_Init();
    sei();
    usart_init();
    while (1) {
        if(on == 1) {
            next_color();
        }
    }
    return 0;
}
